@@ -25,11 +25,9 @@ from typing import (
 
 import pytest
 from outcome import Outcome
-from pytest import MonkeyPatch, WarningsRecorder
 
 import trio
 import trio.testing
-from trio._channel import MemorySendChannel
 from trio.abc import Instrument
 
 from ..._util import signal_raise
@@ -38,6 +36,8 @@ from .tutil import buggy_pypy_asyncgens, gc_collect_harder, restore_unraisableho
 if TYPE_CHECKING:
     from typing_extensions import TypeAlias
 
+    from trio._channel import MemorySendChannel
+
 T = TypeVar("T")
 InHost: TypeAlias = Callable[[object], None]
 
@@ -170,17 +170,16 @@ def test_guest_is_initialized_when_start_returns() -> None:
     assert res == "ok"
     assert set(record) == {"system task ran", "main task ran", "run_sync_soon cb ran"}
 
+    class BadClock:
+        def start_clock(self) -> NoReturn:
+            raise ValueError("whoops")
+
+    def after_start_never_runs() -> None:  # pragma: no cover
+        pytest.fail("shouldn't get here")
+
     # Errors during initialization (which can only be TrioInternalErrors)
     # are raised out of start_guest_run, not out of the done_callback
     with pytest.raises(trio.TrioInternalError):
-
-        class BadClock:
-            def start_clock(self) -> NoReturn:
-                raise ValueError("whoops")
-
-        def after_start_never_runs() -> None:  # pragma: no cover
-            pytest.fail("shouldn't get here")
-
         trivial_guest_run(
             trio_main, clock=BadClock(), in_host_after_start=after_start_never_runs
         )
@@ -526,7 +525,7 @@ def test_guest_mode_on_asyncio() -> None:
 
 
 def test_guest_mode_internal_errors(
-    monkeypatch: MonkeyPatch, recwarn: WarningsRecorder
+    monkeypatch: pytest.MonkeyPatch, recwarn: pytest.WarningsRecorder
 ) -> None:
     with monkeypatch.context() as m:
 
@@ -657,9 +656,6 @@ def test_guest_mode_asyncgens() -> None:
     # Ensure we don't pollute the thread-level context if run under
     # an asyncio without contextvars support (3.6)
     context = contextvars.copy_context()
-    if TYPE_CHECKING:
-        aiotrio_run(trio_main, host_uses_signal_set_wakeup_fd=True)
-    # this type error is a bug in typeshed or mypy, as it's equivalent to the above line
-    context.run(aiotrio_run, trio_main, host_uses_signal_set_wakeup_fd=True)  # type: ignore[arg-type]
+    context.run(aiotrio_run, trio_main, host_uses_signal_set_wakeup_fd=True)
 
     assert record == {("asyncio", "asyncio"), ("trio", "trio")}
