@@ -1,40 +1,64 @@
 # Generic stream tests
+from __future__ import annotations
 
-from contextlib import contextmanager
 import random
+from contextlib import contextmanager, suppress
+from typing import TYPE_CHECKING, Awaitable, Callable, Generic, Tuple, TypeVar
 
-from .. import _core
+from .. import CancelScope, _core
+from .._abc import AsyncResource, HalfCloseableStream, ReceiveStream, SendStream, Stream
 from .._highlevel_generic import aclose_forcefully
-from .._abc import SendStream, ReceiveStream, Stream, HalfCloseableStream
 from ._checkpoints import assert_checkpoints
 
+if TYPE_CHECKING:
+    from collections.abc import Generator
+    from types import TracebackType
 
-class _ForceCloseBoth:
-    def __init__(self, both):
-        self._both = list(both)
+    from typing_extensions import ParamSpec, TypeAlias
 
-    async def __aenter__(self):
-        return self._both
+    ArgsT = ParamSpec("ArgsT")
 
-    async def __aexit__(self, *args):
+Res1 = TypeVar("Res1", bound=AsyncResource)
+Res2 = TypeVar("Res2", bound=AsyncResource)
+StreamMaker: TypeAlias = Callable[[], Awaitable[Tuple[Res1, Res2]]]
+
+
+class _ForceCloseBoth(Generic[Res1, Res2]):
+    def __init__(self, both: tuple[Res1, Res2]) -> None:
+        self._first, self._second = both
+
+    async def __aenter__(self) -> tuple[Res1, Res2]:
+        return self._first, self._second
+
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        traceback: TracebackType | None,
+    ) -> None:
         try:
-            await aclose_forcefully(self._both[0])
+            await aclose_forcefully(self._first)
         finally:
-            await aclose_forcefully(self._both[1])
+            await aclose_forcefully(self._second)
 
 
+# This is used in this file instead of pytest.raises in order to avoid a dependency
+# on pytest, as the check_* functions are publicly exported.
 @contextmanager
-def _assert_raises(exc):
+def _assert_raises(exc: type[BaseException]) -> Generator[None, None, None]:
     __tracebackhide__ = True
     try:
         yield
     except exc:
         pass
     else:
-        raise AssertionError("expected exception: {}".format(exc))
+        raise AssertionError(f"expected exception: {exc}")
 
 
-async def check_one_way_stream(stream_maker, clogged_stream_maker):
+async def check_one_way_stream(
+    stream_maker: StreamMaker[SendStream, ReceiveStream],
+    clogged_stream_maker: StreamMaker[SendStream, ReceiveStream] | None,
+) -> None:
     """Perform a number of generic tests on a custom one-way stream
     implementation.
 
@@ -57,18 +81,18 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
         assert isinstance(s, SendStream)
         assert isinstance(r, ReceiveStream)
 
-        async def do_send_all(data):
-            with assert_checkpoints():
-                assert await s.send_all(data) is None
+        async def do_send_all(data: bytes | bytearray | memoryview) -> None:
+            with assert_checkpoints():  # We're testing that it doesn't return anything.
+                assert await s.send_all(data) is None  # type: ignore[func-returns-value]
 
-        async def do_receive_some(*args):
+        async def do_receive_some(max_bytes: int | None = None) -> bytes | bytearray:
             with assert_checkpoints():
-                return await r.receive_some(*args)
+                return await r.receive_some(max_bytes)
 
-        async def checked_receive_1(expected):
+        async def checked_receive_1(expected: bytes) -> None:
             assert await do_receive_some(1) == expected
 
-        async def do_aclose(resource):
+        async def do_aclose(resource: AsyncResource) -> None:
             with assert_checkpoints():
                 await resource.aclose()
 
@@ -77,7 +101,7 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
             nursery.start_soon(do_send_all, b"x")
             nursery.start_soon(checked_receive_1, b"x")
 
-        async def send_empty_then_y():
+        async def send_empty_then_y() -> None:
             # Streams should tolerate sending b"" without giving it any
             # special meaning.
             await do_send_all(b"")
@@ -104,7 +128,7 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
         with _assert_raises(ValueError):
             await r.receive_some(0)
         with _assert_raises(TypeError):
-            await r.receive_some(1.5)
+            await r.receive_some(1.5)  # type: ignore[arg-type]
         # it can also be missing or None
         async with _core.open_nursery() as nursery:
             nursery.start_soon(do_send_all, b"x")
@@ -123,7 +147,9 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
         # for send_all to wait until receive_some is called to run, though; a
         # stream doesn't *have* to have any internal buffering. That's why we
         # start a concurrent receive_some call, then cancel it.)
-        async def simple_check_wait_send_all_might_not_block(scope):
+        async def simple_check_wait_send_all_might_not_block(
+            scope: CancelScope,
+        ) -> None:
             with assert_checkpoints():
                 await s.wait_send_all_might_not_block()
             scope.cancel()
@@ -136,7 +162,7 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
 
         # closing the r side leads to BrokenResourceError on the s side
         # (eventually)
-        async def expect_broken_stream_on_send():
+        async def expect_broken_stream_on_send() -> None:
             with _assert_raises(_core.BrokenResourceError):
                 while True:
                     await do_send_all(b"x" * 100)
@@ -179,11 +205,11 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
 
     async with _ForceCloseBoth(await stream_maker()) as (s, r):
         # if send-then-graceful-close, receiver gets data then b""
-        async def send_then_close():
+        async def send_then_close() -> None:
             await do_send_all(b"y")
             await do_aclose(s)
 
-        async def receive_send_then_close():
+        async def receive_send_then_close() -> None:
             # We want to make sure that if the sender closes the stream before
             # we read anything, then we still get all the data. But some
             # streams might block on the do_send_all call. So we let the
@@ -216,10 +242,8 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
         # after the sender does a forceful close, the receiver might either
         # get BrokenResourceError or a clean b""; either is OK. Not OK would be
         # if it freezes, or returns data.
-        try:
+        with suppress(_core.BrokenResourceError):
             await checked_receive_1(b"")
-        except _core.BrokenResourceError:
-            pass
 
     # cancelled aclose still closes
     async with _ForceCloseBoth(await stream_maker()) as (s, r):
@@ -248,9 +272,13 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
     #     https://github.com/python-trio/trio/issues/77
     async with _ForceCloseBoth(await stream_maker()) as (s, r):
 
-        async def expect_cancelled(afn, *args):
+        async def expect_cancelled(
+            afn: Callable[ArgsT, Awaitable[object]],
+            *args: ArgsT.args,
+            **kwargs: ArgsT.kwargs,
+        ) -> None:
             with _assert_raises(_core.Cancelled):
-                await afn(*args)
+                await afn(*args, **kwargs)
 
         with _core.CancelScope() as scope:
             scope.cancel()
@@ -278,16 +306,16 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
     # check wait_send_all_might_not_block, if we can
     if clogged_stream_maker is not None:
         async with _ForceCloseBoth(await clogged_stream_maker()) as (s, r):
-            record = []
+            record: list[str] = []
 
-            async def waiter(cancel_scope):
+            async def waiter(cancel_scope: CancelScope) -> None:
                 record.append("waiter sleeping")
                 with assert_checkpoints():
                     await s.wait_send_all_might_not_block()
                 record.append("waiter wokeup")
                 cancel_scope.cancel()
 
-            async def receiver():
+            async def receiver() -> None:
                 # give wait_send_all_might_not_block a chance to block
                 await _core.wait_all_tasks_blocked()
                 record.append("receiver starting")
@@ -329,17 +357,18 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
                     nursery.start_soon(s.send_all, b"123")
                     nursery.start_soon(s.send_all, b"123")
 
-        # closing the receiver causes wait_send_all_might_not_block to return
+        # closing the receiver causes wait_send_all_might_not_block to return,
+        # with or without an exception
         async with _ForceCloseBoth(await clogged_stream_maker()) as (s, r):
 
-            async def sender():
+            async def sender() -> None:
                 try:
                     with assert_checkpoints():
                         await s.wait_send_all_might_not_block()
-                except _core.BrokenResourceError:
+                except _core.BrokenResourceError:  # pragma: no cover
                     pass
 
-            async def receiver():
+            async def receiver() -> None:
                 await _core.wait_all_tasks_blocked()
                 await aclose_forcefully(r)
 
@@ -353,12 +382,12 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
             try:
                 with assert_checkpoints():
                     await s.wait_send_all_might_not_block()
-            except _core.BrokenResourceError:
+            except _core.BrokenResourceError:  # pragma: no cover
                 pass
 
         # Check that if a task is blocked in a send-side method, then closing
         # the send stream causes it to wake up.
-        async def close_soon(s):
+        async def close_soon(s: SendStream) -> None:
             await _core.wait_all_tasks_blocked()
             await aclose_forcefully(s)
 
@@ -375,7 +404,10 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
                     await s.wait_send_all_might_not_block()
 
 
-async def check_two_way_stream(stream_maker, clogged_stream_maker):
+async def check_two_way_stream(
+    stream_maker: StreamMaker[Stream, Stream],
+    clogged_stream_maker: StreamMaker[Stream, Stream] | None,
+) -> None:
     """Perform a number of generic tests on a custom two-way stream
     implementation.
 
@@ -390,13 +422,15 @@ async def check_two_way_stream(stream_maker, clogged_stream_maker):
     """
     await check_one_way_stream(stream_maker, clogged_stream_maker)
 
-    async def flipped_stream_maker():
-        return reversed(await stream_maker())
+    async def flipped_stream_maker() -> tuple[Stream, Stream]:
+        return (await stream_maker())[::-1]
+
+    flipped_clogged_stream_maker: Callable[[], Awaitable[tuple[Stream, Stream]]] | None
 
     if clogged_stream_maker is not None:
 
-        async def flipped_clogged_stream_maker():
-            return reversed(await clogged_stream_maker())
+        async def flipped_clogged_stream_maker() -> tuple[Stream, Stream]:
+            return (await clogged_stream_maker())[::-1]
 
     else:
         flipped_clogged_stream_maker = None
@@ -407,14 +441,16 @@ async def check_two_way_stream(stream_maker, clogged_stream_maker):
         assert isinstance(s2, Stream)
 
         # Duplex can be a bit tricky, might as well check it as well
-        DUPLEX_TEST_SIZE = 2 ** 20
-        CHUNK_SIZE_MAX = 2 ** 14
+        DUPLEX_TEST_SIZE = 2**20
+        CHUNK_SIZE_MAX = 2**14
 
         r = random.Random(0)
         i = r.getrandbits(8 * DUPLEX_TEST_SIZE)
         test_data = i.to_bytes(DUPLEX_TEST_SIZE, "little")
 
-        async def sender(s, data, seed):
+        async def sender(
+            s: Stream, data: bytes | bytearray | memoryview, seed: int
+        ) -> None:
             r = random.Random(seed)
             m = memoryview(data)
             while m:
@@ -422,7 +458,7 @@ async def check_two_way_stream(stream_maker, clogged_stream_maker):
                 await s.send_all(m[:chunk_size])
                 m = m[chunk_size:]
 
-        async def receiver(s, data, seed):
+        async def receiver(s: Stream, data: bytes | bytearray, seed: int) -> None:
             r = random.Random(seed)
             got = bytearray()
             while len(got) < len(data):
@@ -437,7 +473,7 @@ async def check_two_way_stream(stream_maker, clogged_stream_maker):
             nursery.start_soon(receiver, s1, test_data[::-1], 2)
             nursery.start_soon(receiver, s2, test_data, 3)
 
-        async def expect_receive_some_empty():
+        async def expect_receive_some_empty() -> None:
             assert await s2.receive_some(10) == b""
             await s2.aclose()
 
@@ -446,7 +482,10 @@ async def check_two_way_stream(stream_maker, clogged_stream_maker):
             nursery.start_soon(s1.aclose)
 
 
-async def check_half_closeable_stream(stream_maker, clogged_stream_maker):
+async def check_half_closeable_stream(
+    stream_maker: StreamMaker[HalfCloseableStream, HalfCloseableStream],
+    clogged_stream_maker: StreamMaker[HalfCloseableStream, HalfCloseableStream] | None,
+) -> None:
     """Perform a number of generic tests on a custom half-closeable stream
     implementation.
 
@@ -465,12 +504,12 @@ async def check_half_closeable_stream(stream_maker, clogged_stream_maker):
         assert isinstance(s1, HalfCloseableStream)
         assert isinstance(s2, HalfCloseableStream)
 
-        async def send_x_then_eof(s):
+        async def send_x_then_eof(s: HalfCloseableStream) -> None:
             await s.send_all(b"x")
             with assert_checkpoints():
                 await s.send_eof()
 
-        async def expect_x_then_eof(r):
+        async def expect_x_then_eof(r: HalfCloseableStream) -> None:
             await _core.wait_all_tasks_blocked()
             assert await r.receive_some(10) == b"x"
             assert await r.receive_some(10) == b""
